/* Copyright 2017, 2018 Kevin Ryde

   This file is part of nautyextra.

   nautyextra is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   nautyextra is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along
   with nautyextra.  See the file COPYING.  If not, see
   <http://www.gnu.org/licenses/>.
*/

#include <nauty/nauty.h>
#include <nauty/nausparse.h>

#ifndef  _NAUTYEXTRA_H_
#define  _NAUTYEXTRA_H_

#ifndef NAUTYEXTRA_TYPE_ULL
#define NAUTYEXTRA_TYPE_ULL   unsigned long long
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* setword with element 0 alone */
#define NAUTYEXTRA_SETWORD_HIGHBIT  ((setword) 1 << (WORDSIZE-1))

NAUTYEXTRA_TYPE_ULL nautyextra_binomial (int n, int m);

void nautyextra_print_sg (const sparsegraph *sg);
void nautyextra_print_vpar (const int *vpar, int n);
char *nautyextra_set_str(const set *ss, int n);

int nautyextra_validate (const graph *g, int m, int n);
int nautyextra_validate_sg (const sparsegraph *sg);
int nautyextra_validate_vpar (const int *vpar, int n);

void nautyextra_ensure_vertex_sg (sparsegraph *sg, int v);
void nautyextra_add_arc_sg (sparsegraph *sg, int u, int v);
void nautyextra_add_edge_sg (sparsegraph *sg, int u, int v);

int nautyextra_is_undirected (const graph *g, int m, int n);
int nautyextra_is_undirected_sg (const sparsegraph *sg);

int nautyextra_num_arcs (const graph *g, int m, int n);
int nautyextra_num_edges_vpar (const int *vpar, int n);
int nautyextra_num_roots_vpar (const int *vpar, int n);

int nautyextra_root_of_vertex_vpar (const int *vpar, int v);
int nautyextra_depth_of_vertex_vpar (const int *vpar, int v);
int nautyextra_is_ancestor_vpar (const int *vpar, int v, int ancestor);
int nautyextra_common_ancestor_vpar (const int *vpar, int u, int v);
int nautyextra_path_length_vpar (const int *vpar, int u, int v);
int nautyextra_path_vertices_vpar (const int *vpar, int u, int v, int *path);

void nautyextra_degrees_array (const graph *g, int m, int n, int *degrees);
void nautyextra_degrees_array_vpar (const int *vpar, int n, int *degrees);
int  nautyextra_subtree_size_vpar (const int *vpar, int n, int v);
void nautyextra_subtree_sizes_array_vpar (const int *vpar, int n, int *sizes);
void nautyextra_num_children_array_vpar (const int *vpar, int n, int *num_children);

struct nautyextra_diameter_info {
  int diameter;
  int radius;
  int end_v1, end_v2;   /* two vertices attaining the diameter */
  int end_ancestor;     /* their common ancestor */
  int num_endpoints;    /* number of endpoint vertices of all diameters */
  int num_vertices;     /* number of vertices on some diameter */
};
int nautyextra_diameter_vpar (const int *vpar, int n);
void nautyextra_diameter_info_vpar (const int *vpar, int n, struct nautyextra_diameter_info *info);
int nautyextra_eccentricity_of_vertex_vpar (const int *vpar, int n, int v);

struct nautyextra_centroid_info {
  int v1;
  int v2;
  int weight;
};
void nautyextra_centroid_info_vpar (const int *vpar, int n, struct nautyextra_centroid_info *info);

int nautyextra_aresame (const graph *g1, const graph *g2, int m, int n);

int nautyextra_claw_search (const graph *g, int m, int n, int (*callback)(const graph*,int,int, int,int,int,int, void *), void *data);
int nautyextra_is_clawfree (const graph *g, int m, int n);
int nautyextra_is_clawfree_vpar (const int *vpar, int n);
NAUTYEXTRA_TYPE_ULL nautyextra_claw_count (const graph *g, int m, int n);
NAUTYEXTRA_TYPE_ULL nautyextra_claw_count_vpar (const int *vpar, int n);

int nautyextra_is_indset (const graph *g, int m, int n, const set *ss);
int nautyextra_is_indset_sg (const sparsegraph *sg, const set *ss);
NAUTYEXTRA_TYPE_ULL nautyextra_indsets_count_sg (const sparsegraph *sg);
NAUTYEXTRA_TYPE_ULL nautyextra_indsets_count_vpar (const int *vpar, int n);

struct nautyextra_indnum_and_count {
  int                  indnum;
  NAUTYEXTRA_TYPE_ULL  count;
};
int nautyextra_indnum_vpar (const int *vpar, int n);
void nautyextra_indnum_and_count (const graph *g, int m, int n, struct nautyextra_indnum_and_count *info);
void nautyextra_indnum_and_count_vpar (const int *vpar, int n, struct nautyextra_indnum_and_count *info);

int nautyextra_has_perfect_matching_vpar (const int *vpar, int n);
int nautyextra_matchnum_vpar (const int *vpar, int n);
struct nautyextra_matchnum_and_count {
  int                  matchnum;
  NAUTYEXTRA_TYPE_ULL  count;
};
void nautyextra_matchnum_and_count_vpar (const int *vpar, int n, struct nautyextra_matchnum_and_count *info);

int nautyextra_is_domset1 (const graph *g, int n, setword ss);
int nautyextra_is_domset_sg (const sparsegraph *sg, const set *ss);
NAUTYEXTRA_TYPE_ULL nautyextra_domsets_count_sg (const sparsegraph *sg);
NAUTYEXTRA_TYPE_ULL nautyextra_domsets_count_vpar (const int *vpar, int n);

int nautyextra_domnum_vpar (const int *vpar, int n);
struct nautyextra_domnum_and_count {
  int                  domnum;
  NAUTYEXTRA_TYPE_ULL  count;
};
void nautyextra_domnum_and_count1 (const graph *g, int n, struct nautyextra_domnum_and_count *info);
void nautyextra_domnum_and_count_sg (const sparsegraph *sg, struct nautyextra_domnum_and_count *info);
void nautyextra_domnum_and_count_vpar (const int *vpar, int n, struct nautyextra_domnum_and_count *info);

int nautyextra_is_minimal_domset1 (const graph *g, int n, setword ss);
int nautyextra_is_minimal_domset_sg (const sparsegraph *sg, const set *ss);
int nautyextra_domset_is_minimal_sg (const sparsegraph *sg, const set *ss);
NAUTYEXTRA_TYPE_ULL nautyextra_minimal_domsets_count1 (const graph *g, int n);
NAUTYEXTRA_TYPE_ULL nautyextra_minimal_domsets_count_sg (const sparsegraph *sg);
NAUTYEXTRA_TYPE_ULL nautyextra_minimal_domsets_count_vpar (const int *vpar, int n);

int nautyextra_is_indomset_sg (const sparsegraph *sg, const set *ss);
NAUTYEXTRA_TYPE_ULL nautyextra_indomsets_count_sg (const sparsegraph *sg);
NAUTYEXTRA_TYPE_ULL nautyextra_indomsets_count_vpar (const int *vpar, int n);
NAUTYEXTRA_TYPE_ULL nautyextra_indomsets_count_tree_max (int n);

struct nautyextra_indomnum_and_count {
  int indomnum;
  NAUTYEXTRA_TYPE_ULL count;
};
int  nautyextra_indomnum_vpar (const int *vpar, int n);
void nautyextra_indomnum_and_count_sg (const sparsegraph *sg, struct nautyextra_indomnum_and_count *info);
void nautyextra_indomnum_and_count_vpar (const int *vpar, int n, struct nautyextra_indomnum_and_count *info);

int nautyextra_is_totdomset_sg (const sparsegraph *sg, const set *ss);
NAUTYEXTRA_TYPE_ULL nautyextra_totdomsets_count_sg (const sparsegraph *sg);

struct nautyextra_totdomnum_and_count {
  int totdomnum;
  NAUTYEXTRA_TYPE_ULL count;
};
void nautyextra_totdomnum_and_count_sg (const sparsegraph *sg, struct nautyextra_totdomnum_and_count *info);
int nautyextra_totdomnum_vpar (const int *vpar, int n);
void nautyextra_totdomnum_and_count_vpar (const int *vpar, int n, struct nautyextra_totdomnum_and_count *info);

void nautyextra_ranperm_vpar (int *perm, int n);
void nautyextra_relabel_swap_vpar (int *vpar, int n, int u, int v);
void nautyextra_relabel_perm_vpar (const int *vpar_in, int n, const int *perm, int *vpar_out);
void nautyextra_ranrelabel_vpar (const int *vpar_in, int n, int *vpar_out);
void nautyextra_reroot_vpar  (int *vpar, int n, int v);
void nautyextra_ranreroot_vpar (int *vpar, int n);

int  nautyextra_subdivide_vpar (int *vpar, int n, int s);

void nautyextra_make_path_vpar (int *vpar, int n);
void nautyextra_make_star_vpar (int *vpar, int n);
void nautyextra_make_binomial_tree_vpar (int *vpar, int n);
void nautyextra_make_random_vpar (int *vpar, int n, int forest);

#if NAUTYEXTRA_WANT_TEST_RANDOM
/* used for a test build, exists only in the test program */
int nautyextra_test_random(int limit);
#endif

#ifdef __cplusplus
}
#endif

#endif
