#!/usr/bin/perl -w

# Copyright 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::BaseCnv;
use Math::BigInt;

use FindBin;
use lib $FindBin::Bin;
use MyAlternate '@A073057_HtoL_state_table',
  '@A073057_LtoH_state_table','@A073057_LtoH_state_to_name';
use MyPrintwrap ':all';
use MyFLAT;
$|=1;


if (@ARGV && $ARGV[0] eq 'compact') {
  print "%o (PARI) { my(table=[";
  my $join = '';
  foreach my $state (1 .. $#A073057_LtoH_state_table - 3) {
    print $join;
    if ($state==9 || $state==19) { print "\n%o     "; }
    print join(',', @{$A073057_LtoH_state_table[$state]});
    $join = ';';
  }
  print "]);\n";
  exit 0;
}


print "% generated by tools/$FindBin::Script\n";

printwrap_indent("% GP-DEFINE  ");
printwrap_gp_matrix([@A073057_HtoL_state_table[1..$#A073057_HtoL_state_table]],
                    "A073057_HtoL_state_table");

@A073057_LtoH_state_to_name == @A073057_LtoH_state_table or die;

foreach my $state (0 .. $#A073057_LtoH_state_table) {
  my $name = $A073057_LtoH_state_to_name[$state] // next;
  print "% GP-DEFINE  A073057_LtoH_$name = $state;\n";
}
printwrap_gp_vector
  ([ map {defined ? "\"$_\"" : "'none"}
     @A073057_LtoH_state_to_name[1..$#A073057_LtoH_state_to_name] ],
   "A073057_LtoH_state_to_name");

print "% GP-DEFINE  A073057_LtoH_state_table = {[\n";
foreach my $state (1 .. $#A073057_LtoH_state_table) {
  my $name = $A073057_LtoH_state_to_name[$state] // 'unused';
  my $row = $A073057_LtoH_state_table[$state];
  my $str = join(',',@$row)
    . ($state < $#A073057_LtoH_state_table ? ';' : ']};');
  printf "%% GP-DEFINE    %-10s  \\\\ %s%d = %s",
    $str, 
    ($state % 4 == 0 ? '' : '  '),
    $state,
    $name;
  if ($row->[0]) {
    print "  -> ",join(',',map {$A073057_LtoH_state_to_name[$_]} @$row);
  }
  print "\n";
}

print "% end generated\n";
exit 0;
