#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;
use Graph::Maker::TwinAlternateAreaTree;

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'TAVertexToUpper';
use MyGraphs 'Graph_subtree_depth';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 8;
if (@ARGV) { $level = shift @ARGV; }

print "% generated by tools/$FindBin::Script\n";

my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                               level => $level,
                               undirected => 1);
{
  my @TAparent;
  my %seen = (0 => 1);
  my @pending = (0);
  while (@pending) {
    my $v = pop @pending;
    foreach my $child (grep {!$seen{$_}++} $graph->neighbours($v)) {
      ### $v
      ### $child
      $TAparent[$child] = $v;
      push @pending, $child;
    }
  }

  $Printwrap = 0;
  printwrap_indent("% GP-DEFINE  ");
  printwrap("TAparent_generated_samples = {[");
  $Printwrap += 2;
  foreach my $n (1 .. $#TAparent) {
    printwrap($TAparent[$n] . ($n<$#TAparent?',':''));
  }
  printwrap_indent("% GP-DEFINE  ");
  printwrap("]};\n");
}

{
  ### TAtospine ...
  my @TAtospine;
  foreach my $c (2**$level - 1,            #  01111111
                 2**($level-1) - 1) {
    printf "%% corner %d = %0*b\n", $c, $level, $c;
    $TAtospine[$c] = TAVertexToUpper($c);  # 110000000
    my %seen = ($c => 1);
    my @pending = ($c);
    while (@pending) {
      my $v = pop @pending;
      foreach my $preceding (grep {!$seen{$_}++} $graph->neighbours($v)) {
        ### $v
        ### $preceding
        $TAtospine[$preceding] = $v;
        if ($preceding != 0) {
          push @pending, $preceding;
        }
      }
    }
  }
  my $limit = 2**($level-2) - 1;
  $Printwrap = 0;
  printwrap_indent("% GP-DEFINE  ");
  printwrap("TAtospine_generated_samples = {[");
  $Printwrap += 2;
  foreach my $n (1 .. $#TAtospine) {
    my $value = $TAtospine[$n] // last;
    printwrap($value . ($n<$#TAtospine?',':''));
  }
  printwrap_indent("% GP-DEFINE  ");
  printwrap("]};\n");
}

print "% end generated\n";
exit 0;
