#!/usr/bin/perl -w

# Copyright 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::BigInt;
use Math::NumSeq::PlanePathTurn;
use Math::PlanePath::RationalsTree;

use Math::PlanePath::GcdRationals;
BEGIN { *gcd = \&Math::PlanePath::GcdRationals::_gcd; }


use FindBin;
use lib $FindBin::Bin;
use MyPrintwrap ':all';

my $len = 10;

print "% generated by tools/$FindBin::Script\n";

my $tree = Math::PlanePath::RationalsTree->new;
my $seq = Math::NumSeq::PlanePathTurn->new(planepath => 'AlternatePaper',
                                           turn_type => 'Right');
printwrap_indent("% GP-DEFINE");
$Printwrap = 2;
printwrap("gen_contfrac_SB = {[");
$Printwrap += 2;
my $n = Math::BigInt->new(2);
OUTER: foreach my $i (0 .. 200) {
  my ($seq_i,$bit) = $seq->next;
  foreach (1,2) {
    my ($x,$y) = $tree->n_to_xy($n);
    $len--;
    if ($len >= 0) {
      printwrap (" [$x;$y]" . ($len ? ',' : ''));
    }
    $n = 2*$n + $bit;
  }
}
printwrap(" ]};\n");
printwrap_newline();

{
  my ($x,$y) = $tree->n_to_xy($n);
  print "% GP-DEFINE  gen_contfrac_SB_big = $x/$y;\n";
}

print "% end generated\n";
exit 0;
