#!/usr/bin/perl -w

# Copyright 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'chamfer';
use MyPrintwrap ':all';


my $start = 13;
if (@ARGV) { $start = shift @ARGV; }

print "    % generated by tools/$FindBin::Script $start\n";
my $path = Math::PlanePath::AlternatePaper->new;

my $end = $start + 10;

print "    \\newcommand\\MyN{$start}\n";
{
  my ($x,$y) = $path->n_to_xy($start);
  print "    \\node (start) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = $path->n_to_xy($end);
  print "    \\coordinate (end) at ($x,$y);\n";
}

$Printwrap = 4;
printwrap("\\draw[->] (start)");
$Printwrap += 2;
foreach my $n ($start+1 .. $end-1) {
  my ($x,$y) = $path->n_to_xy($n);
  printwrap("--($x,$y)");
  if (($n-$start)%2 == 0) {
    printwrap(";");
    printwrap(" \\draw[->] ($x,$y)");
  }
}
{
  my ($x,$y) = $path->n_to_xy($end-1);
  my ($dx,$dy) = $path->n_to_dxdy($end-1);
  printwrap("--($x+\\MyEnd*$dx,$y+\\MyEnd*$dy);\n");
}

print "    % end generated\n";
exit 0;
